package com.bgy.sms.channel.bgy.service.impl;


import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.bgy.sms.channel.bgy.config.BgySMSConfig;
import com.bgy.sms.channel.bgy.dto.CLBizResponse;
import com.bgy.sms.channel.bgy.dto.TemplateNotify;
import com.bgy.sms.channel.bgy.service.BgySmsService;
import com.bgy.sms.channel.bgy.utils.SendSmsUtil;
import com.bgy.sms.channel.dmHub.service.DmHubService;
import com.bgy.sms.config.ResponseCode;
import com.bgy.sms.repository.domain.SmsTemplateInfo;
import com.bgy.sms.service.SmsTemplateService;
import com.bgy.util.HttpUtil;
import com.bgy.util.Md5Util;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


@SuppressWarnings({"DuplicatedCode", "Duplicates"})
@Service
public class BgySmsServiceImpl implements BgySmsService {

    private static final Logger log = LoggerFactory.getLogger(BgySmsServiceImpl.class);
    private static final String SUCCESS = "success";
    private static final String sendSuccessCode = "0";  // 发送响应成功字段
    private static final String createTemplateStatus = "status"; // 响应状态字段
    private static final String retMsg = "msg";                 //  响应描述
    private static final String createTemplateData = "data";    //  创建模板响应数据

    private static final String notificationId = "49"; // 碧桂园通知类短信ID
    private static final String marketingId = "52";    // 碧桂园营销类短信ID

    @Autowired
    private SmsTemplateService templateService;
    @Autowired
    private DmHubService dmHubService;

    @Autowired
    private SmsTemplateService smsTemplateService;

    @Override
    public CLBizResponse sendSms(String mobile, String content, String areaId, String api) throws Exception {
        log.info("进入碧桂园短信发送接口");
        String appId = BgySMSConfig.appId;
        String securityCode = BgySMSConfig.securityCode;
        String url = BgySMSConfig.url;
        if (api == null) {
            api = BgySMSConfig.api;
        }
        Map<String, String> requestParams = new HashMap<>();
        requestParams.put("api", api);
        requestParams.put("appId", appId);
        requestParams.put("security", Md5Util.encrypt(appId + securityCode.toUpperCase()));
        requestParams.put("areaId", areaId);
        requestParams.put("content", content);
        requestParams.put("mobile", mobile);
        log.info("碧桂园短信接口参数：{}", requestParams);
        String retStr = SendSmsUtil.sendSmsByPost(url, JSONObject.toJSONString(requestParams));
        log.info("碧桂园短信接口返回信息：{}", retStr);
        if (retStr == null) {
            return new CLBizResponse(ResponseCode.UPSTREAM_BLANK);
        }
        JSONObject retJson = JSONObject.parseObject(retStr);
        String err = retJson.getString("err");
        String retPack = retJson.getString("package");
        String retCode = retJson.getString("ret");
        if (!sendSuccessCode.equals(retCode)) {
            if (err.contains("成功")) {
                return new CLBizResponse(ResponseCode.SUCCESS);
            }
            return new CLBizResponse(ResponseCode.UPSTREAM_FAIL.getCode(), err);
        } else {
            return new CLBizResponse(ResponseCode.SUCCESS);
        }
    }

    @Override
    public CLBizResponse sendBatchSms(List<JSONObject> data, String areaId, String api, String templateId) throws Exception {
        log.info("进入碧桂园短信发送接口");
        String appId = BgySMSConfig.appId;
        String securityCode = BgySMSConfig.securityCode;
        String url = BgySMSConfig.url;
        Map<String, Object> requestParams = new HashMap<>();
        requestParams.put("api", api);
        requestParams.put("appId", appId);
        requestParams.put("security", Md5Util.encrypt(appId + securityCode.toUpperCase()));
        requestParams.put("areaId", areaId);
        requestParams.put("templateId", templateId);
        requestParams.put("data", data);
        log.info("碧桂园短信接口参数：{}", requestParams);
        String reqStr = JSONObject.toJSONString(requestParams);
        String retStr = SendSmsUtil.sendSmsByPost(url, reqStr);
        log.info("碧桂园短信接口返回信息：{}", retStr);
        if (retStr == null) {
            return new CLBizResponse(ResponseCode.UPSTREAM_BLANK);
        }
        JSONObject retJson = JSONObject.parseObject(retStr);
        String err = retJson.getString("err");
        String retPack = retJson.getString("package");
        String retCode = retJson.getString("ret");
        if (!sendSuccessCode.equals(retCode)) {
            if (err.contains("成功")) {
                return new CLBizResponse(ResponseCode.SUCCESS);
            }
            return new CLBizResponse(ResponseCode.UPSTREAM_FAIL.getCode(), err);
        } else {
            return new CLBizResponse(ResponseCode.SUCCESS);
        }
    }

    /**
     * 处理短信模板的通知
     *
     * @param templateNotify
     */
    @Override
    public void asyncNotify(TemplateNotify templateNotify) {
        log.info("碧桂园短信模板审核通知接口入参：{}", templateNotify);
        //模板流水号
        String templateRecordId = templateNotify.getData();
        String status = templateNotify.getAuditStatus();
        String auditReason = templateNotify.getAuditReason();
        SmsTemplateInfo smsTemplateInfo = smsTemplateService.selectOne(new EntityWrapper<SmsTemplateInfo>().eq("template_record_id", templateRecordId));
        if (smsTemplateInfo == null) {
            throw new RuntimeException(ResponseCode.NO_TEMPLATE.getMsg());
        }

        if ("1".equals(status)) {
            smsTemplateInfo.setStatus("normal");
            smsTemplateInfo.setLastUpdated(new Date());
        } else {
            smsTemplateInfo.setStatus("abnormal");
        }
        smsTemplateInfo.setUpRejectMsg(auditReason);
        smsTemplateInfo.setLastUpdated(new Date());
        smsTemplateService.updateById(smsTemplateInfo);
    }


    @Override
    public CLBizResponse createTemplate(String content, String account, String templateType, String pkID) throws Exception {
        log.info("进入碧桂园创建模板接口");
        String appId = BgySMSConfig.appId;
        String securityCode = BgySMSConfig.securityCode;
        Map<String, String> param = new HashMap<>();
        param.put("api", "AddTemplate");//TODO
        param.put("appid", appId);
        param.put("security", Md5Util.encrypt(appId + securityCode.toUpperCase()));
        param.put("content", content);
        param.put("areaid", "XXJSB");
        param.put("pkID", pkID);
        param.put("slxtID", account);
        param.put("templateType", templateType);

        log.info("碧桂园创建模板接口参数：{}", param);
        String retStr = HttpUtil.sendPost(BgySMSConfig.url, JSONObject.toJSONString(param));
        log.info("碧桂园创建模板接口返回信息：{}", retStr);
        if (retStr == null) {
            return new CLBizResponse(ResponseCode.UPSTREAM_BLANK);
        }
        JSONObject retJson = JSONObject.parseObject(retStr);
        String err = retJson.getString("err");
        String retPack = retJson.getString("package");
        String retCode = retJson.getString("ret");
        if (!sendSuccessCode.equals(retCode)) {
            if (err.contains("成功")) {
                JSONObject Json = JSONObject.parseObject(retPack);
                String data = Json.getString("data");
                return new CLBizResponse(ResponseCode.SUCCESS.getCode(), data);
            }
            return new CLBizResponse(ResponseCode.UPSTREAM_FAIL.getCode(), err);
        } else {
            JSONObject Json = JSONObject.parseObject(retPack);
            String data = Json.getString("data");
            return new CLBizResponse(ResponseCode.SUCCESS.getCode(), data);
        }
    }


    public static void main(String[] args) {


        String appId = BgySMSConfig.appId;
        String a = Md5Util.encrypt(2021 + "930844c7-7985-435b-af47-142b59b299c3".toUpperCase());

        System.out.println(a);
        System.out.println(appId);
        if (1 == 1) {
            return;
        }
//        String areaId = BgySMSConfig.areaId;
        String securityCode = BgySMSConfig.securityCode;
        String url = BgySMSConfig.url;
        String api = BgySMSConfig.api;
        Map<String, String> requestParams = new HashMap<>();
        requestParams.put("api", "SendYXSMS");
        requestParams.put("appId", "");
        requestParams.put("security", "");
        requestParams.put("areaId", "XXJSB");
        requestParams.put("content", "测试模板内容,占位符测试：11，再来一个：22！退订回复TD【碧桂园】");
        requestParams.put("mobile", "18088834894");
        log.info("碧桂园短信接口参数：{}", requestParams);
        String retStr = SendSmsUtil.sendSmsByPost("https://xstest.bgy.com.cn/ApiBgyTest/Open/Sms.ashx", JSONObject.toJSONString(requestParams));
        log.info("碧桂园短信接口返回信息：{}", retStr);
    }
}
